'
' Read a series of named BMP files from an SD Card formatted as FAT16
' Display the bitmaps on the Amicus18 Clour Graphic LCD shield
'
' Written by Les Johnson. September 2011
'
    Declare Optimiser_Level = 3                 ' Maximum optimisation
'
' SD Card Declarations
' Default settings for the Amicus Touch Screen Colour LCD Shield
'
    $define SD_SDO PORTC.5                      ' SPI DO to SD card DI pin
    $define SD_SDI PORTC.4                      ' SPI DI to SD card DO pin
    $define SD_CLK PORTC.3                      ' SPI CLK to SD card CLK pin
    $define SD_CS PORTA.4                       ' SPI CS to SD card CS pin (active low)
    $define SD_Power PORTA.5                    ' Enable/Disable the SD card's VDD (active low)
    $define SD_Detect PORTA.2                   ' Detect if the SD card is inserted
   
    Include "ILI9320.inc"                       ' Load the ILI9320 graphic LCD routines into the program
    Include "FAT16.inc"                         ' Load the FAT16 reading routines into the program

    Dim bLoop As Byte                           ' Used to read and discard the bmp file's header
    
    Dim AmountOfFiles As Word
    Dim FileSelect As Word
    Dim FileNowDisplaying As Word

'----------------------------------------------------------------------------------------------------------
    GoTo Main                                    ' Jump over any subroutines
'----------------------------------------------------------------------------------------------------------
' Display an image on the graphic LCD
' Input     : None
' Output    : None
' Notes     : The file containing the image must already have been opened
'           : The .bmp file must be format 16-bit RGB565 without a colour lookup table
'
Display_Image:
'
' Discard the BMP header information
'
    bLoop = 0
    Repeat
        File_ReadByte()
        Inc bLoop
    Until bLoop >= $46
'
' Read and display the bmp file
'                                       
    Glcd_WriteReg($60,%1010011100000000)                    ' Reverse the direction of the scan, otherwise images are mirrored 
    Glcd_SetWindow(0, 0, 239, 319)                          ' Create a window the full size of the LCD's screen
    Clear Glcd_CS                                           ' Enable the LCD
    hSetCGRAM(0,0)                                          ' Move to the correct location
    While 1 = 1
        wPixelColour = File_ReadWord()                      ' Read 16-bits from the file and use it as the pixel's colour       
        hWriteDataWord(wPixelColour)                        ' Write the 16-bits directly to the LCD
        If FAT_bResponse = cFAT_EndOfFile Then Break        ' Exit the loop when the End-Of-File is reached
    Wend
    Set Glcd_CS                                             ' Disable the LCD
    DelayMS 4096                                            ' Keep the image on the LCD for a while
    Return
'----------------------------------------------------------------------------------------------------------
' The main program loop starts here
' Open a sequence of .bmp files and display them on the Amicus18 Graphic LCD shield
'
Main:
    Glcd_Cls(clWhite)                                               ' Clear the LCD with the colour white   
    If FAT_Init() = True Then                                       ' Did the media initialise?
        While 1 = 1                                                 ' Yes. So create an infinite loop
            If File_Open("Shed.bmp") <> cFAT_FileNotFound Then      ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("1.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("2.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("3.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("4.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("5.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("6.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("7.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("8.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("9.bmp") <> cFAT_FileNotFound Then         ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("10.bmp") <> cFAT_FileNotFound Then        ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("11.bmp") <> cFAT_FileNotFound Then        ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("12.bmp") <> cFAT_FileNotFound Then        ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("13.bmp") <> cFAT_FileNotFound Then        ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
            If File_Open("14.bmp") <> cFAT_FileNotFound Then        ' Did the file open correctly?
                GoSub Display_Image                                 ' Yes. So display it on the LCD
            EndIf
        Wend
    Else
        HRSOut "FAT Init Fail\r"
    EndIf
    Stop

