'
' Stylophone type electronic organ with vibrato effect
' With the keyboard displayed on the LCD
'
' Connect a speaker from PORTA.0 to Ground
'
' Written by Les Johnson. September 2011
'  
    Declare Optimiser_Level = 3                 ' Maximum optimisation
    Include "ILI9320.inc"                       ' Load the graphic LCD routines into the program
    Include "TouchScreen.inc"                   ' Load the touch screen routines into the program
    Include "Timers.inc"                        ' Load the timer macros into the program
    
    On_Hardware_Interrupt GoTo Int_Tone         ' Point to the interrupt handler routine
'
' Keyboard image positions
'        
    Symbol cKey_YStartPos = 3   
    Symbol cKey_Height = 52
    Symbol cKey_Width = 26
    Symbol cKey_XPos = 238 - cKey_Height
    Symbol cKey_Ypos = 2 
'
' Slide switch image positions
'    
    Symbol cSwitch_Xstart = 50
    Symbol cSwitch_Ystart = 260
    Symbol cSwitch_Width = 20
    Symbol cSwitch_Height = 50   
    Symbol cBut_Width = cSwitch_Width
    Symbol cBut_Height = cSwitch_Height / 2   
'
' Frequency values for each note on the keyboard
'  
    Symbol cA1       = 403    
    Symbol cA1_Sharp = 379
    Symbol cB1       = 359
    Symbol cC1       = 339
    Symbol cC1_Sharp = 326
    Symbol cD1       = 302
    Symbol cD1_Sharp = 285
    Symbol cE1       = 269
    Symbol cF1       = 254
    Symbol cF1_Sharp = 239
    Symbol cG1       = 225
    Symbol cG1_Sharp = 213
    Symbol cA2       = 201
    Symbol cA2_Sharp = 189
    Symbol cB2       = 179
    Symbol cC2       = 169
    Symbol cC2_Sharp = 162
    Symbol cD2       = 150
    Symbol cD2_Sharp = 142
    Symbol cE2       = 134
'
' Note generating
'           
    Dim FreguencyValue As Word System  
    Dim wNoteCounter As Word System
    Dim wNote As Word System
    Dim ReTimer As Word System
'
' Keyboard positioning
'    
    Dim KeyCount As Byte   
    Dim Key_Xpos As Byte
    Dim Key_Ypos As Word
'
' Vibrato triangle wave
'    
    Dim TriValue As Byte
    Dim tTriIncrement As Bit
    Dim tVibrato As Bit
    
    Symbol Note_Pin = LATA.1                        ' Connect a speaker to this pin
    
'-------------------------------------------------------------------------------------------     
    GoTo Main                                       ' Jump over the subroutines
'------------------------------------------------------------------------------------------- 
    Include "Courier_Bold_14.inc"                   ' Load the font that is being used
'------------------------------------------------------------------------------------------- 
' Interrupt handler
' This will create a frequency from a pin of the microcontroller
'
Int_Tone:
    If INTCONbits_TMR0IF = True Then
        WriteTimer0(ReTimer)                        ' Load Timer0 with a preset value
        If wNote > 0 Then                           ' Is there a note to play?
            Inc wNoteCounter                        ' Yes. So increment the note counter        
            If wNoteCounter >= wNote Then           ' Is it time to toggle the pin?
                Btg Note_Pin                        ' Toggle the pin
                Clear wNoteCounter                  ' Clear the note counter
            EndIf       
        EndIf
        INTCONbits_TMR0IF = 0                       ' Clear the Timer0 Overflow flag
    EndIf
    Retfie Fast
'------------------------------------------------------------------------------------------- 
' Draw the image of a slide switch
'
Draw_Switch:
    Glcd_PenColor(clGray0)
    Glcd_SolidRectangle(cSwitch_Xstart, cSwitch_Ystart, (cSwitch_Xstart + cSwitch_Height), (cSwitch_Ystart + cSwitch_Width))       
    Glcd_PenColor(clBlack)
    Glcd_Rectangle(cSwitch_Xstart, cSwitch_Ystart, cSwitch_Height, cSwitch_Width)
    Return
'------------------------------------------------------------------------------------------- 
' Draw the button within the slide switch to indicate Off
' This will also disable the vibrato
'
Switch_Off:
    GoSub Draw_Switch
    Glcd_PenColor(clBlack)
    Glcd_SolidRectangle((cSwitch_Xstart + 1), (cSwitch_Ystart + 1), ((cSwitch_Xstart - 1) + cBut_Height), ((cSwitch_Ystart - 1) + cSwitch_Width))
    tVibrato = False
    Return
'------------------------------------------------------------------------------------------- 
' Draw the button within the slide switch to indicate On
' This will also enable the vibrato
'
Switch_On:
    GoSub Draw_Switch
    Glcd_PenColor(clBlack)
    Glcd_SolidRectangle(((cSwitch_Xstart + 1) + cBut_Height), (cSwitch_Ystart + 1), ((cSwitch_Xstart + cSwitch_Height) - 1), ((cSwitch_Ystart - 1) + cSwitch_Width))
    tVibrato = True
    Return     
'------------------------------------------------------------------------------------------- 
' Draw the keyboard on the LCD
' 
Draw_Keyboard:
    Glcd_PenColor(clBrown)
    Glcd_SolidRectangle(((cKey_XPos - cKey_Height) - 3), 0, ((cKey_XPos + cKey_Height) + 1), 319)
'
' Draw the major keys of the keyboard
'    
    Key_Ypos = cKey_YStartPos
    For KeyCount = 11 To 0 Step -1
        Glcd_PenColor(clGray0)
        Glcd_SolidRectangle(cKey_XPos, Key_Ypos, (cKey_XPos + cKey_Height), (Key_Ypos + cKey_Width))       
        Glcd_PenColor(clBlack)
        Glcd_Rectangle(cKey_XPos, Key_Ypos, cKey_Height, cKey_Width)
        Key_Ypos = Key_Ypos + cKey_Width
    Next
'
' Draw the sharp keys of the keyboard
'    
    For KeyCount = 0 To 7
        Key_Ypos = CRead16 KeySpaces[KeyCount]      
        Glcd_PenColor(clGray0)
        Glcd_SolidRectangle((cKey_XPos - cKey_Height) , Key_Ypos, cKey_XPos, (Key_Ypos + cKey_Width))      
        Glcd_PenColor(clBlack)
        Glcd_Rectangle((cKey_XPos - cKey_Height), Key_Ypos, cKey_Height, cKey_Width)          
    Next
    Return 
'------------------------------------------------------------------------------------------- 
' Detect a touch on the keyboard 
' Input     : None
' Output    : FreguencyValue holds the note to play for a key pressed on the keyboard
'           : Otherwise holds 0 if no key pressed
' Notes     : Also checks for the vibrato switch
'
Detect_Keypress:
    FreguencyValue = 0                                      ' Default to no key pressed
    If Touch_Active() = tStylusOn Then                      ' Is the stylus touching the screen?                   
        Touch_Get()                                         ' Yes. So get its X and Y pixel positions            
        If Touch_TableHotSpot(Hotspot_Note_A1) = True Then
            FreguencyValue = cA1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_B1) = True Then 
            FreguencyValue = cB1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_C1) = True Then
            FreguencyValue = cC1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_D1) = True Then 
            FreguencyValue = cD1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_E1) = True Then  
            FreguencyValue = cE1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_F1) = True Then
            FreguencyValue = cF1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_G1) = True Then  
            FreguencyValue = cG1
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_A2) = True Then   
            FreguencyValue = cA2
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_B2) = True Then
            FreguencyValue = cB2
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_C2) = True Then
            FreguencyValue = cC2
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_D2) = True Then
            FreguencyValue = cD2
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_E2) = True Then 
            FreguencyValue = cE2
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_A1_Sharp) = True Then 
            FreguencyValue = cA1_Sharp
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_C1_Sharp) = True Then
            FreguencyValue = cC1_Sharp
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_D1_Sharp) = True Then
            FreguencyValue = cD1_Sharp
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_F1_Sharp) = True Then
            FreguencyValue = cF1_Sharp
            Return
        EndIf  
        If Touch_TableHotSpot(Hotspot_Note_G1_Sharp) = True Then 
            FreguencyValue = cG1_Sharp
            Return
        EndIf 
        If Touch_TableHotSpot(Hotspot_Note_A2_Sharp) = True Then
            FreguencyValue = cA2_Sharp
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_C2_Sharp) = True Then
            FreguencyValue = cC2_Sharp
            Return
        EndIf
        If Touch_TableHotSpot(Hotspot_Note_D2_Sharp) = True Then
            FreguencyValue = cD2_Sharp
            Return
        EndIf
        '
        ' Check for the vibrato slide switch
        '
        If Touch_TableHotSpot(HotSpot_SwitchOn) = True Then
            GoSub Switch_On
            Return
        EndIf 
        If Touch_TableHotSpot(HotSpot_SwitchOff) = True Then
            GoSub Switch_Off
            Return
        EndIf
    EndIf
    Return
'------------------------------------------------------------------------------------------- 
' The main program starts here
'   
Main:  
    Glcd_Cls(clGreen)                                   ' Clear the LCD with the colour green
    Glcd_SetFont(Courier_Bold_14)                       ' Choose the font for texts
    Glcd_PaperColor(clGreen)                            ' Background for texts is green
    
    tVerticalPrint = True                               ' Enable vertical printing
    ReTimer = 65440
    Output Note_Pin                                     ' Make the sound pin an output
    Clear wNoteCounter                                  ' Reset the Note counter
'
' Configure Timer0 for:
'                       Clear TMR0L and TMR0H registers
'                       Interrupt on Timer0 overflow
'                       16-bit operation
'                       Internal clock source
'                       1:1 Prescaler
'
    OpenTimer0(TIMER_INT_ON & T0_16BIT & T0_SOURCE_INT & T0_PS_1_1)
   
    INTCONbits_GIE = 1                                  ' Enable Global Interrupts       
'
' Display the vibrato switch and its texts
'            
    Glcd_Cursor((cSwitch_Xstart - 20), (cSwitch_Ystart + 38))
    Glcd_PenColor(clBlack)
    Print "Vibrato"
    Glcd_Cursor((cSwitch_Xstart + 2), (cSwitch_Ystart - cSwitch_Width))
    Print "Off"
    Glcd_Cursor((cSwitch_Xstart + cSwitch_Height - 18), (cSwitch_Ystart - cSwitch_Width))
    Print "On"
    GoSub Switch_Off                                    ' Display the vibrato switch 
    
    tTriIncrement = True                                ' Triangle will start ascending
    TriValue = 0                                        ' Reset the triangle value
    GoSub Draw_Keyboard                                 ' Create and display the keyboard
    While 1 = 1                                         ' Create an infinite loop
        GoSub Detect_Keypress                           ' Detect a stylus on the LCD
        wNote = FreguencyValue                          ' Transfer the frequency value for the interrupt
        If tVibrato = True Then                         ' Do we require vibrato?
            If tTriIncrement = True Then                ' Yes. So start ascending the amplitude
                Inc TriValue                            ' Increase the amplitude of the triangel wave
                If TriValue >= 4 Then tTriIncrement = False  ' Stop ascending when maximum reached   
            Else                                        ' Otherwise...
                Dec TriValue                            ' Decrease the amplitude of the triangel wave
                If TriValue = 0 Then tTriIncrement = True ' Stop descending when 0 reached 
            EndIf 
            INTCONbits_GIE = 0                          ' Disable the interrupt    
            ReTimer = TriValue + 65440                  ' Add the triangle wave to the interrupt's interval
            INTCONbits_GIE = 1                          ' Re-Enable the interrupt
        Else                                            ' Otherwise...
            INTCONbits_GIE = 0                          ' Disable the interrupt
            ReTimer = 65440
            INTCONbits_GIE = 1                          ' Re-Enable the interrupt  
        EndIf        
    Wend                                                ' Do it forever
    
'------------------------------------------------------------------------------------------- 
' Keyboard Y locations for drawing sharp keys
'
KeySpaces:
    CData As Word ((cKey_Width / 2) + cKey_YStartPos),_ 
                  ((cKey_Width / 2) + cKey_YStartPos) + cKey_Width,_
                  ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 3),_
                  ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 4),_
                  ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 5),_
                  ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 7),_
                  ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 8),_
                  ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 10)
                  
'------------------------------------------------------------------------------------------- 
' Vibrato slide switch hotspot touch area
'
' Format is: Xstart, Ystart, Xend, Yend  
'  
HotSpot_SwitchOn:
    CData Byte ((cSwitch_Xstart + 1) + cBut_Height),_
          Word (cSwitch_Ystart + 1),_
          Byte ((cSwitch_Xstart + cSwitch_Height) - 1),_
          Word ((cSwitch_Ystart - 1) + cSwitch_Width)
HotSpot_SwitchOff:
    CData Byte (cSwitch_Xstart + 1),_
          Word (cSwitch_Ystart + 1),_
          Byte ((cSwitch_Xstart - 1) + cBut_Height),_
          Word ((cSwitch_Ystart - 1) + cSwitch_Width) 
'------------------------------------------------------------------------------------------- 
' Hotspot tables for keyboard touch recognition
'                  
' Format is: Xstart, Ystart, Xend, Yend
' 
Hotspot_Note_A1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 12)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 12))                     
Hotspot_Note_B1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 11)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 11))           
Hotspot_Note_C1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 10)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 10))            
Hotspot_Note_D1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 9)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 9)) 
Hotspot_Note_E1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 8)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 8)) 
Hotspot_Note_F1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 7)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 7)) 
Hotspot_Note_G1:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 6)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 6))               
Hotspot_Note_A2:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 5)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 5))     
Hotspot_Note_B2:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 4)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 4))     
Hotspot_Note_C2:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 3)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 3)) 
Hotspot_Note_D2:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 2)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 2))     
Hotspot_Note_E2:
    CData Byte cKey_XPos, Word ((cKey_YStartPos + (cKey_Width * 1)) - cKey_Width),_
          Byte (cKey_XPos + cKey_Height), Word (cKey_YStartPos + (cKey_Width * 1))

Hotspot_Note_A1_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 10),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 10) + cKey_Width
Hotspot_Note_C1_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 8),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 8) + cKey_Width    
Hotspot_Note_D1_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 7),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 7) + cKey_Width
Hotspot_Note_F1_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 5),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 5) + cKey_Width 
Hotspot_Note_G1_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 4),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 4) + cKey_Width 
Hotspot_Note_A2_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 3),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 3) + cKey_Width  
Hotspot_Note_C2_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 1),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + (cKey_Width * 1) + cKey_Width    
Hotspot_Note_D2_Sharp:
    CData Byte (cKey_XPos - cKey_Height), Word ((cKey_Width / 2) + cKey_YStartPos),_
          Byte cKey_XPos, Word ((cKey_Width / 2) + cKey_YStartPos) + cKey_Width     
    
           
